//  Copyright 2016 root.ext@gmail.com
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import 'coder.dart';
import 'native.dart';
import 'meta.dart';
import 'model/library_meta.dart';
import 'model/method_meta.dart';
import 'model/argument.dart';

class DynamicLibrary<T> {
  final int STACK_SIZE = 256;

  Type _type;
  LibraryMeta _library;

  DynamicLibrary([Resolver resolver]) {
    _type = T;

    assert(_type != dynamic, 'dynamic is not supported');
    //print('_type $_type');

    _library = getLibraryMeta(_type, resolver);    
    //print('library pointer ${_library.pointer}');   
  }

  dynamic noSuchMethod(Invocation invocation) {
    
    var methodMeta = getMethodMeta(invocation, _library);
    assert(methodMeta.pointer != NULL_POINTER, 'cannot find method ${invocation.memberName}');
    //print('method pointer ${methodMeta.pointer}');

    var vm = createCallVM(STACK_SIZE);
    assert(vm != NULL_POINTER, 'cannot create vm');

    var refArgs = new List<Argument>();
    var nativeParams = encodeParams(invocation, methodMeta.params, refArgs);
    var types = paramTypes(invocation, methodMeta);
    assert(nativeParams.length == types.length, 'cannot encode params');
    pushParameters(vm, types, nativeParams);
    var result = _call(methodMeta, vm);
    decodeParams(invocation, methodMeta.params, refArgs);

    freeCallVM(vm);
    return result;
  }
  
  dynamic _call(MethodMeta methodMeta, int vm) {
    var result = callFunction(vm, methodMeta.pointer, methodMeta.returnParam.type);
    return result;
  }
}
