//  Copyright 2016 root.ext@gmail.com
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import 'dart:convert';
import 'dart:typed_data';

int getInt(ByteData view, int size, int offset) {
  switch (size) {
    case 1:
      return view.getInt8(offset);
    case 2:
      return view.getInt16(offset, Endian.host);
    case 4:
      return view.getInt32(offset, Endian.host);
    case 8:
      return view.getInt64(offset, Endian.host);
    default:
      throw new ArgumentError("Wrong int ${#size}");
  }
}

double getFloat(ByteData view, int size, int offset) {
  switch (size) {
    case 4:
      return view.getFloat32(offset);
    case 8:
      return view.getFloat64(offset);
    default:
      throw new ArgumentError("Wrong float ${#size}");
  }
}

void setInt(ByteData view, int size, int offset, int value) {
  switch (size) {
    case 1:
      view.setInt8(offset, value);
      break;
    case 2:
      view.setInt16(offset, value, Endian.host);
      break;
    case 4:
      view.setInt32(offset, value, Endian.host);
      break;
    case 8:
      view.setInt64(offset, value, Endian.host);
      break;
    default:
      throw new ArgumentError("Wrong int ${#size}");
  }
}

void setFloat(ByteData view, int size, int offset, double value) {
  switch (size) {
    case 4:
      view.setFloat32(offset, value);
      break;
    case 8:
      view.setFloat64(offset, value);
      break;
    default:
      throw new ArgumentError("Wrong float ${#size}");
  }
}


TypedData fromIntList(List object, int size) {
  TypedData data;

  switch (size) {
    case 1:
      data = new Uint8List.fromList(object);
      break;
    case 2:
      data = new Uint16List.fromList(object);
      break;
    case 4:
      data = new Uint32List.fromList(object);
      break;
    case 8:
      data = new Uint64List.fromList(object);
      break;
  }
  return data;
}

List toIntList(TypedData data, int size) {
  List list;

  switch (size) {
    case 1:
      list = (data as Uint8List).toList();
      break;
    case 2:
      list = (data as Uint16List).toList();
      break;
    case 4:
      list = (data as Uint32List).toList();
      break;
    case 8:
      list = (data as Uint64List).toList();
      break;
  }
  return list;
}


TypedData fromFloatList(List object, int size) {
  TypedData data;

  switch (size) {
    case 4:
      data = new Float32List.fromList(object);
      break;
    case 8:
      data = new Float64List.fromList(object);
      break;
  }
  return data;
}

List toFloatList(TypedData data, int size) {
  List list;

  switch (size) {
    case 4:
      list = (data as Float32List).toList();
      break;
    case 8:
      list = (data as Float64List).toList();
      break;
  }
  return list;
}

void copyByDest(ByteData src, ByteData dest, int offset) {
  for (var i = 0; i < dest.lengthInBytes; i++) {
    dest.setUint8(i, src.getUint8(i + offset));
  }
}

void copyBySrc(ByteData src, ByteData dest, int offset) {
  for (var i = 0; i < src.lengthInBytes; i++) {
    dest.setUint8(i + offset, src.getUint8(i));
  }
}

TypedData stringToData(String value) {
  return new Uint8List.fromList(utf8.encode(value));
}

String dataToString(TypedData data) {
  return utf8.decode(new Uint8List.view(data.buffer));
}
