//  Copyright 2016 root.ext@gmail.com
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import 'dart:mirrors';

import 'native.dart';
import 'types.dart';

List<int> _sizes;

int ctypeToSize(int type, {ref = false}) {
  if (_sizes == null) { _sizes = getSizes(); }
  switch (type) {
    case C_CHAR:
    case C_UCHAR:
      return _sizes[0];
    case C_SHORT:
    case C_USHORT:
      return _sizes[1];
    case C_INT:
    case C_UINT:
      return _sizes[2];
    case C_LONG:
    case C_ULONG:
      return _sizes[3];
    case C_LONGLONG:
    case C_ULONGLONG:
      return _sizes[4];
    case C_BOOL:
      return _sizes[5];
    case C_FLOAT:
      return _sizes[6];
    case C_DOUBLE:
      return _sizes[7];
    case C_POINTER:
      return _sizes[8];
    case C_STRING:
      return _sizes[8];
    case C_STRUCT:
      return ref ? _sizes[8] : 0;
    case C_INT8:
      return 1;
    case C_INT16:
      return 2;
    case C_INT32:
      return 4;
    case C_INT64:
      return 8;
    default:
      return 0;
  }
}

bool isInteger(int type) {
  return (type >= C_CHAR) && (type <= C_UINT64) && (type != C_BOOL) ||
      (type == C_POINTER);
}

bool isDecimal(int type) {
  return (type >= C_FLOAT) && (type <= C_DOUBLE);
}

int dartTypeToCType(TypeMirror type) {
  if (type == reflectClass(int)) {
    return C_INT;
  } else if (type == reflectClass(double)) {
    return C_DOUBLE;
  } else if (type == reflectClass(bool)) {
    return C_BOOL;
  } else if (type == reflectClass(String)) {
    return C_STRING;
  } else {
    return VOID;
  }
}

int align(int offset, int size) {
  return offset % size != 0 ? (size - offset % size) : 0;
}

String trimCString(String str) {
  var index = str.indexOf('\x00');
  return str.substring(0, index > 0 ? index : str.length);
}
